use std::sync::Arc;

use axum::extract::Path;
use axum::{extract::State, response::IntoResponse, Form};

use crate::admin::domain::{AdminApp, AdminModel};
use crate::admin::state;
use crate::service::templates;
use serde::{Deserialize, Serialize};

#[derive(Deserialize)]
pub struct Question {
    question: String,
}

#[derive(Deserialize)]
pub struct ExampleData {
    title: String,
    content: String,
}

#[derive(Serialize, Deserialize)]
pub struct AdminRequest {
    pub path: String,
}

#[derive(Deserialize, Serialize)]
pub struct AdminContext {
    pub language_code: Option<String>,
    pub language_bidi: Option<bool>,
    pub user: Option<String>, // Todo: user type
    pub site_url: Option<String>,
    pub docsroot: Option<String>,
    pub messages: Vec<String>, // Todo: message type
    pub title: Option<String>,
    pub subtitle: Option<String>,
    pub content: String,

    pub request: AdminRequest,
    pub available_apps: Vec<AdminApp>,
}

impl Default for AdminContext {
    fn default() -> Self {
        AdminContext {
            language_code: Some("en-us".to_string()), // Default language code
            language_bidi: Some(false),               // Default language bidi
            user: None, //UserType::default(),                // Assuming UserType has a Default impl
            site_url: None,
            docsroot: None,
            messages: Vec::new(), // Empty vector for messages
            title: None,
            subtitle: None,
            content: String::new(), // Empty string for content
            available_apps: Vec::new(),
            request: AdminRequest {
                path: "".to_owned(),
            },
        }
    }
}

pub async fn index(
    templates: State<templates::Templates>,
    registry: State<Arc<state::AdminRegistry>>,
) -> impl IntoResponse {
    templates.render_html(
        "admin/index.html",
        AdminContext {
            available_apps: registry.get_apps(),
            ..Default::default()
        },
    )
}

// Index Action is POST to the index site. We can anchor some general business code here.
pub async fn index_action(Form(example_data): Form<ExampleData>) -> impl IntoResponse {
    "There is your answer!".to_owned()
}

pub async fn list_app(
    templates: State<templates::Templates>,
    Path(app_key): Path<String>,
) -> impl IntoResponse {
    templates.render_html("admin/app_list.html", ())
}

// List Items renders the entire list item page.
pub async fn list_item_collection(
    templates: State<templates::Templates>,
    Path((app_key, model_key)): Path<(String, String)>,
) -> impl IntoResponse {
    templates.render_html("admin/items/list_items.html", ())
}

// Items Action is a POST to an item list. By default these are actions, that work on a list of items as input.
pub async fn item_collection_action(
    Form(question): Form<Question>,
    Path((app_key, model_key)): Path<(String, String)>,
) -> impl IntoResponse {
    "There is your answer!".to_owned()
}

// Item Details shows one single dataset.
pub async fn item_details(
    templates: State<templates::Templates>,
    Path((app_key, model_key, id)): Path<(String, String, String)>,
) -> impl IntoResponse {
    templates.render_html("admin/items/item_detail.html", ())
}

// Item Action allows running an action on one single dataset.
pub async fn item_action(
    Form(question): Form<Question>,
    Path((app_key, model_key, model_id)): Path<(String, String, String)>,
) -> impl IntoResponse {
    "There is your answer!".to_owned()
}
